@@ -16,7 +16,7 @@ import traceback
 from coverage import env
 from coverage.disposition import FileDisposition, disposition_init
 from coverage.exceptions import CoverageException, PluginError
-from coverage.files import TreeMatcher, FnmatchMatcher, ModuleMatcher
+from coverage.files import TreeMatcher, GlobMatcher, ModuleMatcher
 from coverage.files import prep_patterns, find_python_files, canonical_filename
 from coverage.misc import sys_modules_saved
 from coverage.python import source_for_file, source_for_morf
@@ -189,9 +189,10 @@ def add_coverage_paths(paths):
 class InOrOut:
     """Machinery for determining what files to measure."""
 
-    def __init__(self, warn, debug):
+    def __init__(self, warn, debug, include_namespace_packages):
         self.warn = warn
         self.debug = debug
+        self.include_namespace_packages = include_namespace_packages
 
         # The matchers for should_trace.
         self.source_match = None
@@ -260,10 +261,10 @@ class InOrOut:
                 self.pylib_match = TreeMatcher(self.pylib_paths, "pylib")
                 debug(f"Python stdlib matching: {self.pylib_match!r}")
         if self.include:
-            self.include_match = FnmatchMatcher(self.include, "include")
+            self.include_match = GlobMatcher(self.include, "include")
             debug(f"Include matching: {self.include_match!r}")
         if self.omit:
-            self.omit_match = FnmatchMatcher(self.omit, "omit")
+            self.omit_match = GlobMatcher(self.omit, "omit")
             debug(f"Omit matching: {self.omit_match!r}")
 
         self.cover_match = TreeMatcher(self.cover_paths, "coverage")
@@ -565,14 +566,17 @@ class InOrOut:
         Yield the file path, and the plugin name that handles the file.
 
         """
-        py_files = ((py_file, None) for py_file in find_python_files(src_dir))
+        py_files = (
+            (py_file, None) for py_file in
+            find_python_files(src_dir, self.include_namespace_packages)
+        )
         plugin_files = self._find_plugin_files(src_dir)
 
         for file_path, plugin_name in itertools.chain(py_files, plugin_files):
             file_path = canonical_filename(file_path)
             if self.omit_match and self.omit_match.match(file_path):
                 # Turns out this file was omitted, so don't pull it back
-                # in as unexecuted.
+                # in as un-executed.
                 continue
             yield file_path, plugin_name
 
@@ -591,7 +595,7 @@ class InOrOut:
             'source_match', 'source_pkgs_match',
             'include_match', 'omit_match',
             'cover_match', 'pylib_match', 'third_match',
-            ]
+        ]
 
         for matcher_name in matcher_names:
             matcher = getattr(self, matcher_name)
