@@ -23,7 +23,8 @@ from coverage.files import abs_file, relative_filename
 from coverage.misc import import_local_file
 
 from tests.coveragetest import CoverageTest, TESTS_DIR, UsingModulesMixin
-from tests.helpers import assert_count_equal, assert_coverage_warnings
+from tests.goldtest import contains, doesnt_contain
+from tests.helpers import arcz_to_arcs, assert_count_equal, assert_coverage_warnings
 from tests.helpers import change_dir, nice_file, os_sep
 
 BAD_SQLITE_REGEX = r"file( is encrypted or)? is not a database"
@@ -71,7 +72,6 @@ class ApiTest(CoverageTest):
         assert missing == [1]
 
     def test_filenames(self):
-
         self.make_file("mymain.py", """\
             import mymod
             a = 1
@@ -478,8 +478,11 @@ class ApiTest(CoverageTest):
 
     def test_ordered_combine(self):
         # https://github.com/nedbat/coveragepy/issues/649
-        # The order of the [paths] setting matters
-        def make_data_file():
+        # The order of the [paths] setting used to matter. Now the
+        # resulting path must exist, so the order doesn't matter.
+        def make_files():
+            self.make_file("plugins/p1.py", "")
+            self.make_file("girder/g1.py", "")
             self.make_data_file(
                 basename=".coverage.1",
                 lines={
@@ -498,7 +501,7 @@ class ApiTest(CoverageTest):
             return filenames
 
         # Case 1: get the order right.
-        make_data_file()
+        make_files()
         self.make_file(".coveragerc", """\
             [paths]
             plugins =
@@ -510,8 +513,8 @@ class ApiTest(CoverageTest):
             """)
         assert get_combined_filenames() == {'girder/g1.py', 'plugins/p1.py'}
 
-        # Case 2: get the order wrong.
-        make_data_file()
+        # Case 2: get the order "wrong".
+        make_files()
         self.make_file(".coveragerc", """\
             [paths]
             girder =
@@ -521,7 +524,7 @@ class ApiTest(CoverageTest):
                 plugins/
                 ci/girder/plugins/
             """)
-        assert get_combined_filenames() == {'girder/g1.py', 'girder/plugins/p1.py'}
+        assert get_combined_filenames() == {'girder/g1.py', 'plugins/p1.py'}
 
     def test_warnings(self):
         self.make_file("hello.py", """\
@@ -817,7 +820,7 @@ class IncludeOmitTestsMixin(UsingModulesMixin, CoverageTest):
         self.filenames_in(result, "p1a p1b p2a p2b othera otherb osa osb")
         self.filenames_not_in(result, "p1c")
         # Because there was no source= specified, we don't search for
-        # unexecuted files.
+        # un-executed files.
 
     def test_include(self):
         result = self.coverage_usepkgs(include=["*/p1a.py"])
@@ -866,7 +869,11 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
         # Since we need to import from there, we also add it to the beginning
         # of sys.path.
 
-        shutil.copytree(nice_file(TESTS_DIR, "modules"), "tests_dir_modules")
+        shutil.copytree(
+            nice_file(TESTS_DIR, "modules"),
+            "tests_dir_modules",
+            ignore=shutil.ignore_patterns("__pycache__"),
+        )
         sys.path.insert(0, abs_file("tests_dir_modules"))
 
     def coverage_usepkgs(self, **kwargs):
@@ -898,7 +905,7 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
         lines = self.coverage_usepkgs(source=["pkg1"])
         self.filenames_in(lines, "p1a p1b")
         self.filenames_not_in(lines, "p2a p2b othera otherb osa osb")
-        # Because source= was specified, we do search for unexecuted files.
+        # Because source= was specified, we do search for un-executed files.
         assert lines['p1c'] == 0
 
     def test_source_package_as_dir(self):
@@ -907,13 +914,13 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
         lines = self.coverage_usepkgs(source=["pkg1"])
         self.filenames_in(lines, "p1a p1b")
         self.filenames_not_in(lines, "p2a p2b othera otherb osa osb")
-        # Because source= was specified, we do search for unexecuted files.
+        # Because source= was specified, we do search for un-executed files.
         assert lines['p1c'] == 0
 
     def test_source_package_dotted_sub(self):
         lines = self.coverage_usepkgs(source=["pkg1.sub"])
         self.filenames_not_in(lines, "p2a p2b othera otherb osa osb")
-        # Because source= was specified, we do search for unexecuted files.
+        # Because source= was specified, we do search for un-executed files.
         assert lines['runmod3'] == 0
 
     def test_source_package_dotted_p1b(self):
@@ -925,7 +932,7 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
         # https://github.com/nedbat/coveragepy/issues/218
         # Used to be if you omitted something executed and inside the source,
         # then after it was executed but not recorded, it would be found in
-        # the search for unexecuted files, and given a score of 0%.
+        # the search for un-executed files, and given a score of 0%.
 
         # The omit arg is by path, so need to be in the modules directory.
         os.chdir("tests_dir_modules")
@@ -955,7 +962,7 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
         lines = self.coverage_usepkgs(source_pkgs=["pkg1"])
         self.filenames_in(lines, "p1a p1b")
         self.filenames_not_in(lines, "p2a p2b othera otherb osa osb ambiguous")
-        # Because source= was specified, we do search for unexecuted files.
+        # Because source= was specified, we do search for un-executed files.
         assert lines['p1c'] == 0
 
 
@@ -1193,6 +1200,10 @@ class RelativePathTest(CoverageTest):
             cov.save()
             shutil.move(glob.glob(".coverage.*")[0], "..")
 
+        self.make_file("foo.py", "a = 1")
+        self.make_file("bar.py", "a = 1")
+        self.make_file("modsrc/__init__.py", "x = 1")
+
         self.make_file(".coveragerc", """\
             [run]
             relative_files = true
@@ -1205,10 +1216,6 @@ class RelativePathTest(CoverageTest):
         cov.combine()
         cov.save()
 
-        self.make_file("foo.py", "a = 1")
-        self.make_file("bar.py", "a = 1")
-        self.make_file("modsrc/__init__.py", "x = 1")
-
         cov = coverage.Coverage()
         cov.load()
         files = cov.get_data().measured_files()
@@ -1236,6 +1243,38 @@ class RelativePathTest(CoverageTest):
         self.assert_file_count(".coverage.*", 0)
         self.assert_exists(".coverage")
 
+    def test_files_up_one_level(self):
+        # https://github.com/nedbat/coveragepy/issues/1280
+        self.make_file("src/mycode.py", """\
+            def foo():
+                return 17
+            """)
+        self.make_file("test/test_it.py", """\
+            from src.mycode import foo
+            assert foo() == 17
+            """)
+        self.make_file("test/.coveragerc", """\
+            [run]
+            parallel = True
+            relative_files = True
+
+            [paths]
+            source =
+                ../src/
+                */src
+            """)
+        os.chdir("test")
+        sys.path.insert(0, "..")
+        cov1 = coverage.Coverage()
+        self.start_import_stop(cov1, "test_it")
+        cov1.save()
+        cov2 = coverage.Coverage()
+        cov2.combine()
+        cov3 = coverage.Coverage()
+        cov3.load()
+        report = self.get_report(cov3)
+        assert self.last_line_squeezed(report) == "TOTAL 4 0 100%"
+
 
 class CombiningTest(CoverageTest):
     """More tests of combining data."""
@@ -1327,7 +1366,7 @@ class CombiningTest(CoverageTest):
 
         # Make bogus data files.
         self.make_file(".coverage.bad1", "This isn't a coverage data file.")
-        self.make_file(".coverage.bad2", "This isn't a coverage data file.")
+        self.make_file(".coverage.bad2", "This isn't a coverage data file either.")
 
         # Combine the parallel coverage data files into .coverage, but nothing is readable.
         cov = coverage.Coverage()
@@ -1421,3 +1460,146 @@ class CombiningTest(CoverageTest):
         # After combining, the .coverage file & the original combined file should still be there.
         self.assert_exists(".coverage")
         self.assert_file_count(".coverage.*", 2)
+
+
+class ReportMapsPathsTest(CoverageTest):
+    """Check that reporting implicitly maps paths."""
+
+    def make_files(self, data, settings=False):
+        """Create the test files we need for line coverage."""
+        src = """\
+            if VER == 1:
+                print("line 2")
+            if VER == 2:
+                print("line 4")
+            if VER == 3:
+                print("line 6")
+            """
+        self.make_file("src/program.py", src)
+        self.make_file("ver1/program.py", src)
+        self.make_file("ver2/program.py", src)
+
+        if data == "line":
+            self.make_data_file(
+                lines={
+                    abs_file("ver1/program.py"): [1, 2, 3, 5],
+                    abs_file("ver2/program.py"): [1, 3, 4, 5],
+                }
+            )
+        else:
+            self.make_data_file(
+                arcs={
+                    abs_file("ver1/program.py"): arcz_to_arcs(".1 12 23 35 5."),
+                    abs_file("ver2/program.py"): arcz_to_arcs(".1 13 34 45 5."),
+                }
+            )
+
+        if settings:
+            self.make_file(".coveragerc", """\
+                [paths]
+                source =
+                    src
+                    ver1
+                    ver2
+                """)
+
+    def test_map_paths_during_line_report_without_setting(self):
+        self.make_files(data="line")
+        cov = coverage.Coverage()
+        cov.load()
+        cov.report(show_missing=True)
+        expected = textwrap.dedent(os_sep("""\
+            Name              Stmts   Miss  Cover   Missing
+            -----------------------------------------------
+            ver1/program.py       6      2    67%   4, 6
+            ver2/program.py       6      2    67%   2, 6
+            -----------------------------------------------
+            TOTAL                12      4    67%
+            """))
+        assert expected == self.stdout()
+
+    def test_map_paths_during_line_report(self):
+        self.make_files(data="line", settings=True)
+        cov = coverage.Coverage()
+        cov.load()
+        cov.report(show_missing=True)
+        expected = textwrap.dedent(os_sep("""\
+            Name             Stmts   Miss  Cover   Missing
+            ----------------------------------------------
+            src/program.py       6      1    83%   6
+            ----------------------------------------------
+            TOTAL                6      1    83%
+            """))
+        assert expected == self.stdout()
+
+    def test_map_paths_during_branch_report_without_setting(self):
+        self.make_files(data="arcs")
+        cov = coverage.Coverage(branch=True)
+        cov.load()
+        cov.report(show_missing=True)
+        expected = textwrap.dedent(os_sep("""\
+            Name              Stmts   Miss Branch BrPart  Cover   Missing
+            -------------------------------------------------------------
+            ver1/program.py       6      2      6      3    58%   1->3, 4, 6
+            ver2/program.py       6      2      6      3    58%   2, 3->5, 6
+            -------------------------------------------------------------
+            TOTAL                12      4     12      6    58%
+            """))
+        assert expected == self.stdout()
+
+    def test_map_paths_during_branch_report(self):
+        self.make_files(data="arcs", settings=True)
+        cov = coverage.Coverage(branch=True)
+        cov.load()
+        cov.report(show_missing=True)
+        expected = textwrap.dedent(os_sep("""\
+            Name             Stmts   Miss Branch BrPart  Cover   Missing
+            ------------------------------------------------------------
+            src/program.py       6      1      6      1    83%   6
+            ------------------------------------------------------------
+            TOTAL                6      1      6      1    83%
+            """))
+        assert expected == self.stdout()
+
+    def test_map_paths_during_annotate(self):
+        self.make_files(data="line", settings=True)
+        cov = coverage.Coverage()
+        cov.load()
+        cov.annotate()
+        self.assert_exists(os_sep("src/program.py,cover"))
+        self.assert_doesnt_exist(os_sep("ver1/program.py,cover"))
+        self.assert_doesnt_exist(os_sep("ver2/program.py,cover"))
+
+    def test_map_paths_during_html_report(self):
+        self.make_files(data="line", settings=True)
+        cov = coverage.Coverage()
+        cov.load()
+        cov.html_report()
+        contains("htmlcov/index.html", os_sep("src/program.py"))
+        doesnt_contain("htmlcov/index.html", os_sep("ver1/program.py"), os_sep("ver2/program.py"))
+
+    def test_map_paths_during_xml_report(self):
+        self.make_files(data="line", settings=True)
+        cov = coverage.Coverage()
+        cov.load()
+        cov.xml_report()
+        contains("coverage.xml", "src/program.py")
+        doesnt_contain("coverage.xml", "ver1/program.py", "ver2/program.py")
+
+    def test_map_paths_during_json_report(self):
+        self.make_files(data="line", settings=True)
+        cov = coverage.Coverage()
+        cov.load()
+        cov.json_report()
+        def os_sepj(s):
+            return os_sep(s).replace("\\", r"\\")
+        contains("coverage.json", os_sepj("src/program.py"))
+        doesnt_contain("coverage.json", os_sepj("ver1/program.py"), os_sepj("ver2/program.py"))
+
+    def test_map_paths_during_lcov_report(self):
+        self.make_files(data="line", settings=True)
+        cov = coverage.Coverage()
+        cov.load()
+        cov.lcov_report()
+        contains("coverage.lcov", os_sep("src/program.py"))
+        doesnt_contain("coverage.lcov", os_sep("ver1/program.py"), os_sep("ver2/program.py"))
