@@ -43,8 +43,8 @@ class BaseCmdLineTest(CoverageTest):
     )
     _defaults.Coverage().report(
         ignore_errors=None, include=None, omit=None, morfs=[],
-        show_missing=None, skip_covered=None, contexts=None, skip_empty=None, precision=None,
-        sort=None,
+        show_missing=None, skip_covered=None, contexts=None, skip_empty=None,
+        precision=None, sort=None, output_format=None,
     )
     _defaults.Coverage().xml_report(
         ignore_errors=None, include=None, omit=None, morfs=[], outfile=None,
@@ -585,6 +585,11 @@ class CmdLineTest(BaseCmdLineTest):
             cov.load()
             cov.report(show_missing=None)
             """)
+        self.cmd_executes("report --format=markdown", """\
+            cov = Coverage()
+            cov.load()
+            cov.report(output_format="markdown")
+            """)
 
     def test_run(self):
         # coverage run [-p] [-L] [--timid] MODULE.py [ARG1 ARG2 ...]
@@ -1027,6 +1032,12 @@ class CmdLineStdoutTest(BaseCmdLineTest):
         assert "fooey" in err
         assert "help" in err
 
+    def test_option_error(self):
+        self.command_line("run --fooey", ret=ERR)
+        err = self.stderr()
+        assert "fooey" in err
+        assert "help" in err
+
     def test_doc_url(self):
         assert __url__.startswith("https://coverage.readthedocs.io")
 
