@@ -67,7 +67,12 @@ def DebugCoverageData(*args, **kwargs):
     lines in our coverage reports.
     """
     assert "debug" not in kwargs
-    debug = DebugControlString(options=["dataio", "dataop", "sql"])
+    options = ["dataio", "dataop", "sql"]
+    if kwargs:
+        # There's no reason kwargs should imply sqldata debugging.
+        # This is a way to get a mix of debug options across the tests.
+        options.extend(["sqldata"])
+    debug = DebugControlString(options=options)
     return CoverageData(*args, debug=debug, **kwargs)
 
 
@@ -113,6 +118,13 @@ class CoverageDataTest(CoverageTest):
     def test_empty_data_is_false(self):
         covdata = DebugCoverageData()
         assert not covdata
+        self.assert_doesnt_exist(".coverage")
+
+    def test_empty_data_is_false_when_read(self):
+        covdata = DebugCoverageData()
+        covdata.read()
+        assert not covdata
+        self.assert_doesnt_exist(".coverage")
 
     def test_line_data_is_true(self):
         covdata = DebugCoverageData()
@@ -160,15 +172,17 @@ class CoverageDataTest(CoverageTest):
         assert_line_counts(covdata, SUMMARY_3_4)
         assert_measured_files(covdata, MEASURED_FILES_3_4)
 
-    def test_cant_add_arcs_with_lines(self):
-        covdata = DebugCoverageData()
+    @pytest.mark.parametrize("klass", [CoverageData, DebugCoverageData])
+    def test_cant_add_arcs_with_lines(self, klass):
+        covdata = klass()
         covdata.add_lines(LINES_1)
         msg = "Can't add branch measurements to existing line data"
         with pytest.raises(DataError, match=msg):
             covdata.add_arcs(ARCS_3)
 
-    def test_cant_add_lines_with_arcs(self):
-        covdata = DebugCoverageData()
+    @pytest.mark.parametrize("klass", [CoverageData, DebugCoverageData])
+    def test_cant_add_lines_with_arcs(self, klass):
+        covdata = klass()
         covdata.add_arcs(ARCS_3)
         msg = "Can't add line measurements to existing branch data"
         with pytest.raises(DataError, match=msg):
@@ -774,14 +788,16 @@ class CoverageDataFilesTest(CoverageTest):
 
         self.assert_file_count(".coverage.*", 2)
 
+        self.make_file("a.py", "")
+        self.make_file("sub/b.py", "")
+        self.make_file("template.html", "")
         covdata3 = DebugCoverageData()
         aliases = PathAliases()
         aliases.add("/home/ned/proj/src/", "./")
         aliases.add(r"c:\ned\test", "./")
         combine_parallel_data(covdata3, aliases=aliases)
         self.assert_file_count(".coverage.*", 0)
-        # covdata3 hasn't been written yet. Should this file exist or not?
-        #self.assert_exists(".coverage")
+        self.assert_exists(".coverage")
 
         apy = canonical_filename('./a.py')
         sub_bpy = canonical_filename('./sub/b.py')
@@ -864,6 +880,53 @@ class CoverageDataFilesTest(CoverageTest):
         # "no such table: meta"
         covdata2.add_lines(LINES_1)
 
+    @pytest.mark.parametrize(
+        "dpart, fpart",
+        [
+            ("", "[b-a]"),
+            ("[3-1]", ""),
+            ("[3-1]", "[b-a]"),
+        ],
+    )
+    def test_combining_with_crazy_filename(self, dpart, fpart):
+        dirname = f"py{dpart}"
+        basename = f"{dirname}/.coverage{fpart}"
+        os.makedirs(dirname)
+
+        covdata1 = CoverageData(basename=basename, suffix="1")
+        covdata1.add_lines(LINES_1)
+        covdata1.write()
+
+        covdata2 = CoverageData(basename=basename, suffix="2")
+        covdata2.add_lines(LINES_2)
+        covdata2.write()
+
+        covdata3 = CoverageData(basename=basename)
+        combine_parallel_data(covdata3)
+        assert_line_counts(covdata3, SUMMARY_1_2)
+        assert_measured_files(covdata3, MEASURED_FILES_1_2)
+        self.assert_file_count(glob.escape(basename) + ".*", 0)
+
+    def test_meta_data(self):
+        # The metadata written to the data file shouldn't interfere with
+        # hashing to remove duplicates, except for debug=process, which
+        # writes debugging info as metadata.
+        debug = DebugControlString(options=[])
+        covdata1 = CoverageData(basename="meta.1", debug=debug)
+        covdata1.add_lines(LINES_1)
+        covdata1.write()
+        with sqlite3.connect("meta.1") as con:
+            data = sorted(k for (k,) in con.execute("select key from meta"))
+        assert data == ["has_arcs", "version"]
+
+        debug = DebugControlString(options=["process"])
+        covdata2 = CoverageData(basename="meta.2", debug=debug)
+        covdata2.add_lines(LINES_1)
+        covdata2.write()
+        with sqlite3.connect("meta.2") as con:
+            data = sorted(k for (k,) in con.execute("select key from meta"))
+        assert data == ["has_arcs", "sys_argv", "version", "when"]
+
 
 class DumpsLoadsTest(CoverageTest):
     """Tests of CoverageData.dumps and loads."""
@@ -891,3 +954,19 @@ class DumpsLoadsTest(CoverageTest):
         )
         with pytest.raises(DataError, match=msg):
             covdata.loads(bad_data)
+
+
+class NoDiskTest(CoverageTest):
+    """Tests of in-memory CoverageData."""
+
+    run_in_temp_dir = False
+
+    def test_updating(self):
+        # https://github.com/nedbat/coveragepy/issues/1323
+        a = CoverageData(no_disk=True)
+        a.add_lines({'foo.py': [10, 20, 30]})
+        assert a.measured_files() == {'foo.py'}
+
+        b = CoverageData(no_disk=True)
+        b.update(a)
+        assert b.measured_files() == {'foo.py'}
