@@ -3,8 +3,10 @@
 
 """Tests for files.py"""
 
+import itertools
 import os
 import os.path
+import re
 from unittest import mock
 
 import pytest
@@ -12,8 +14,8 @@ import pytest
 from coverage import env, files
 from coverage.exceptions import ConfigError
 from coverage.files import (
-    FnmatchMatcher, ModuleMatcher, PathAliases, TreeMatcher, abs_file,
-    actual_path, find_python_files, flat_rootname, fnmatches_to_regex,
+    GlobMatcher, ModuleMatcher, PathAliases, TreeMatcher, abs_file,
+    actual_path, find_python_files, flat_rootname, globs_to_regex,
 )
 from tests.coveragetest import CoverageTest
 
@@ -67,7 +69,7 @@ class FilesTest(CoverageTest):
         assert files.canonical_filename('sub/proj1/file1.py') == self.abs_path('file1.py')
 
     @pytest.mark.parametrize(
-        ["curdir", "sep"], [
+        "curdir, sep", [
             ("/", "/"),
             ("X:\\", "\\"),
         ]
@@ -79,6 +81,21 @@ class FilesTest(CoverageTest):
                     files.set_relative_directory()
                     assert files.relative_directory() == curdir
 
+    @pytest.mark.parametrize(
+        "to_make, to_check, answer", [
+            ("a/b/c/foo.py", "a/b/c/foo.py", True),
+            ("a/b/c/foo.py", "a/b/c/bar.py", False),
+            ("src/files.zip", "src/files.zip/foo.py", True),
+            ("src/files.egg", "src/files.egg/foo.py", True),
+            ("src/files.pex", "src/files.pex/foo.py", True),
+            ("src/files.zip", "src/morefiles.zip/foo.py", False),
+            ("src/files.pex", "src/files.pex/zipfiles/files.zip/foo.py", True),
+        ]
+    )
+    def test_source_exists(self, to_make, to_check, answer):
+        self.make_file(to_make, "")
+        assert files.source_exists(to_check) == answer
+
 
 @pytest.mark.parametrize("original, flat", [
     ("abc.py", "abc_py"),
@@ -104,53 +121,139 @@ def test_flat_rootname(original, flat):
     assert flat_rootname(original) == flat
 
 
+def globs_to_regex_params(
+    patterns, case_insensitive=False, partial=False, matches=(), nomatches=(),
+):
+    """Generate parameters for `test_globs_to_regex`.
+
+    `patterns`, `case_insensitive`, and `partial` are arguments for
+    `globs_to_regex`.  `matches` is a list of strings that should match, and
+    `nomatches` is a list of strings that should not match.
+
+    Everything is yielded so that `test_globs_to_regex` can call
+    `globs_to_regex` once and check one result.
+
+    """
+    pat_id = "|".join(patterns)
+    for text in matches:
+        yield pytest.param(
+            patterns, case_insensitive, partial, text, True,
+            id=f"{pat_id}:ci{case_insensitive}:par{partial}:{text}:match",
+        )
+    for text in nomatches:
+        yield pytest.param(
+            patterns, case_insensitive, partial, text, False,
+            id=f"{pat_id}:ci{case_insensitive}:par{partial}:{text}:nomatch",
+        )
+
 @pytest.mark.parametrize(
-        "patterns, case_insensitive, partial," +
-            "matches," +
-            "nomatches",
-[
-    (
-        ["abc", "xyz"], False, False,
+    "patterns, case_insensitive, partial, text, result",
+    list(itertools.chain.from_iterable([
+        globs_to_regex_params(
             ["abc", "xyz"],
-            ["ABC", "xYz", "abcx", "xabc", "axyz", "xyza"],
-    ),
-    (
-        ["abc", "xyz"], True, False,
-            ["abc", "xyz", "Abc", "XYZ", "AbC"],
-            ["abcx", "xabc", "axyz", "xyza"],
-    ),
-    (
-        ["abc/hi.py"], True, False,
-            ["abc/hi.py", "ABC/hi.py", r"ABC\hi.py"],
-            ["abc_hi.py", "abc/hi.pyc"],
-    ),
-    (
-        [r"abc\hi.py"], True, False,
-            [r"abc\hi.py", r"ABC\hi.py"],
-            ["abc/hi.py", "ABC/hi.py", "abc_hi.py", "abc/hi.pyc"],
-    ),
-    (
-        ["abc/*/hi.py"], True, False,
-            ["abc/foo/hi.py", "ABC/foo/bar/hi.py", r"ABC\foo/bar/hi.py"],
-            ["abc/hi.py", "abc/hi.pyc"],
-    ),
-    (
-        ["abc/[a-f]*/hi.py"], True, False,
-            ["abc/foo/hi.py", "ABC/foo/bar/hi.py", r"ABC\foo/bar/hi.py"],
-            ["abc/zoo/hi.py", "abc/hi.py", "abc/hi.pyc"],
-    ),
-    (
-        ["abc/"], True, True,
-            ["abc/foo/hi.py", "ABC/foo/bar/hi.py", r"ABC\foo/bar/hi.py"],
-            ["abcd/foo.py", "xabc/hi.py"],
-    ),
+            matches=["abc", "xyz", "sub/mod/abc"],
+            nomatches=[
+                "ABC", "xYz", "abcx", "xabc", "axyz", "xyza", "sub/mod/abcd", "sub/abc/more",
+            ],
+        ),
+        globs_to_regex_params(
+            ["abc", "xyz"], case_insensitive=True,
+            matches=["abc", "xyz", "Abc", "XYZ", "AbC"],
+            nomatches=["abcx", "xabc", "axyz", "xyza"],
+        ),
+        globs_to_regex_params(
+            ["a*c", "x*z"],
+            matches=["abc", "xyz", "xYz", "azc", "xaz", "axyzc"],
+            nomatches=["ABC", "abcx", "xabc", "axyz", "xyza", "a/c"],
+        ),
+        globs_to_regex_params(
+            ["a?c", "x?z"],
+            matches=["abc", "xyz", "xYz", "azc", "xaz"],
+            nomatches=["ABC", "abcx", "xabc", "axyz", "xyza", "a/c"],
+        ),
+        globs_to_regex_params(
+            ["a??d"],
+            matches=["abcd", "azcd", "a12d"],
+            nomatches=["ABCD", "abcx", "axyz", "abcde"],
+        ),
+        globs_to_regex_params(
+            ["abc/hi.py"], case_insensitive=True,
+            matches=["abc/hi.py", "ABC/hi.py", r"ABC\hi.py"],
+            nomatches=["abc_hi.py", "abc/hi.pyc"],
+        ),
+        globs_to_regex_params(
+            [r"abc\hi.py"], case_insensitive=True,
+            matches=[r"abc\hi.py", r"ABC\hi.py", "abc/hi.py", "ABC/hi.py"],
+            nomatches=["abc_hi.py", "abc/hi.pyc"],
+        ),
+        globs_to_regex_params(
+            ["abc/*/hi.py"], case_insensitive=True,
+            matches=["abc/foo/hi.py", r"ABC\foo/hi.py"],
+            nomatches=["abc/hi.py", "abc/hi.pyc", "ABC/foo/bar/hi.py", r"ABC\foo/bar/hi.py"],
+        ),
+        globs_to_regex_params(
+            ["abc/**/hi.py"], case_insensitive=True,
+            matches=[
+                "abc/foo/hi.py", r"ABC\foo/hi.py", "abc/hi.py", "ABC/foo/bar/hi.py",
+                r"ABC\foo/bar/hi.py",
+            ],
+            nomatches=["abc/hi.pyc"],
+        ),
+        globs_to_regex_params(
+            ["abc/[a-f]*/hi.py"], case_insensitive=True,
+            matches=["abc/foo/hi.py", r"ABC\boo/hi.py"],
+            nomatches=[
+                "abc/zoo/hi.py", "abc/hi.py", "abc/hi.pyc", "abc/foo/bar/hi.py",
+                r"abc\foo/bar/hi.py",
+            ],
+        ),
+        globs_to_regex_params(
+            ["abc/[a-f]/hi.py"], case_insensitive=True,
+            matches=["abc/f/hi.py", r"ABC\b/hi.py"],
+            nomatches=[
+                "abc/foo/hi.py", "abc/zoo/hi.py", "abc/hi.py", "abc/hi.pyc", "abc/foo/bar/hi.py",
+                r"abc\foo/bar/hi.py",
+            ],
+        ),
+        globs_to_regex_params(
+            ["abc/"], case_insensitive=True, partial=True,
+            matches=["abc/foo/hi.py", "ABC/foo/bar/hi.py", r"ABC\foo/bar/hi.py"],
+            nomatches=["abcd/foo.py", "xabc/hi.py"],
+        ),
+        globs_to_regex_params(
+            ["*/foo"], case_insensitive=False, partial=True,
+            matches=["abc/foo/hi.py", "foo/hi.py"],
+            nomatches=["abc/xfoo/hi.py"],
+        ),
+        globs_to_regex_params(
+            ["**/foo"],
+            matches=["foo", "hello/foo", "hi/there/foo"],
+            nomatches=["foob", "hello/foob", "hello/Foo"],
+        ),
+    ]))
+)
+def test_globs_to_regex(patterns, case_insensitive, partial, text, result):
+    regex = globs_to_regex(patterns, case_insensitive=case_insensitive, partial=partial)
+    assert bool(regex.match(text)) == result
+
+
+@pytest.mark.parametrize("pattern, bad_word", [
+    ("***/foo.py", "***"),
+    ("bar/***/foo.py", "***"),
+    ("*****/foo.py", "*****"),
+    ("Hello]there", "]"),
+    ("Hello[there", "["),
+    ("Hello+there", "+"),
+    ("{a,b}c", "{"),
+    ("x/a**/b.py", "a**"),
+    ("x/abcd**/b.py", "abcd**"),
+    ("x/**a/b.py", "**a"),
+    ("x/**/**/b.py", "**/**"),
 ])
-def test_fnmatches_to_regex(patterns, case_insensitive, partial, matches, nomatches):
-    regex = fnmatches_to_regex(patterns, case_insensitive=case_insensitive, partial=partial)
-    for s in matches:
-        assert regex.match(s)
-    for s in nomatches:
-        assert not regex.match(s)
+def test_invalid_globs(pattern, bad_word):
+    msg = f"File pattern can't include {bad_word!r}"
+    with pytest.raises(ConfigError, match=re.escape(msg)):
+        globs_to_regex([pattern])
 
 
 class MatcherTest(CoverageTest):
@@ -211,7 +314,7 @@ class MatcherTest(CoverageTest):
         for modulename, matches in matches_to_try:
             assert mm.match(modulename) == matches, modulename
 
-    def test_fnmatch_matcher(self):
+    def test_glob_matcher(self):
         matches_to_try = [
             (self.make_file("sub/file1.py"), True),
             (self.make_file("sub/file2.c"), False),
@@ -219,23 +322,25 @@ class MatcherTest(CoverageTest):
             (self.make_file("sub3/file4.py"), True),
             (self.make_file("sub3/file5.c"), False),
         ]
-        fnm = FnmatchMatcher(["*.py", "*/sub2/*"])
+        fnm = GlobMatcher(["*.py", "*/sub2/*"])
         assert fnm.info() == ["*.py", "*/sub2/*"]
         for filepath, matches in matches_to_try:
             self.assertMatches(fnm, filepath, matches)
 
-    def test_fnmatch_matcher_overload(self):
-        fnm = FnmatchMatcher(["*x%03d*.txt" % i for i in range(500)])
+    def test_glob_matcher_overload(self):
+        fnm = GlobMatcher(["*x%03d*.txt" % i for i in range(500)])
         self.assertMatches(fnm, "x007foo.txt", True)
         self.assertMatches(fnm, "x123foo.txt", True)
         self.assertMatches(fnm, "x798bar.txt", False)
+        self.assertMatches(fnm, "x499.txt", True)
+        self.assertMatches(fnm, "x500.txt", False)
 
-    def test_fnmatch_windows_paths(self):
+    def test_glob_windows_paths(self):
         # We should be able to match Windows paths even if we are running on
         # a non-Windows OS.
-        fnm = FnmatchMatcher(["*/foo.py"])
+        fnm = GlobMatcher(["*/foo.py"])
         self.assertMatches(fnm, r"dir\foo.py", True)
-        fnm = FnmatchMatcher([r"*\foo.py"])
+        fnm = GlobMatcher([r"*\foo.py"])
         self.assertMatches(fnm, r"dir\foo.py", True)
 
 
@@ -250,20 +355,23 @@ class PathAliasesTest(CoverageTest):
 
     run_in_temp_dir = False
 
-    def assert_mapped(self, aliases, inp, out, relative=False):
+    def assert_mapped(self, aliases, inp, out):
         """Assert that `inp` mapped through `aliases` produces `out`.
 
-        `out` is canonicalized first, since aliases produce canonicalized
-        paths by default.
+        If the aliases are not relative, then `out` is canonicalized first,
+        since aliases produce canonicalized paths by default.
 
         """
-        mapped = aliases.map(inp)
-        expected = files.canonical_filename(out) if not relative else out
+        mapped = aliases.map(inp, exists=lambda p: True)
+        if aliases.relative:
+            expected = out
+        else:
+            expected = files.canonical_filename(out)
         assert mapped == expected
 
-    def assert_unchanged(self, aliases, inp):
+    def assert_unchanged(self, aliases, inp, exists=True):
         """Assert that `inp` mapped through `aliases` is unchanged."""
-        assert aliases.map(inp) == inp
+        assert aliases.map(inp, exists=lambda p: exists) == inp
 
     def test_noop(self, rel_yn):
         aliases = PathAliases(relative=rel_yn)
@@ -277,23 +385,62 @@ class PathAliasesTest(CoverageTest):
     def test_wildcard(self, rel_yn):
         aliases = PathAliases(relative=rel_yn)
         aliases.add('/ned/home/*/src', './mysrc')
-        self.assert_mapped(aliases, '/ned/home/foo/src/a.py', './mysrc/a.py', relative=rel_yn)
+        self.assert_mapped(aliases, '/ned/home/foo/src/a.py', './mysrc/a.py')
 
         aliases = PathAliases(relative=rel_yn)
         aliases.add('/ned/home/*/src/', './mysrc')
-        self.assert_mapped(aliases, '/ned/home/foo/src/a.py', './mysrc/a.py', relative=rel_yn)
+        self.assert_mapped(aliases, '/ned/home/foo/src/a.py', './mysrc/a.py')
 
     def test_no_accidental_match(self, rel_yn):
         aliases = PathAliases(relative=rel_yn)
         aliases.add('/home/*/src', './mysrc')
         self.assert_unchanged(aliases, '/home/foo/srcetc')
 
-    def test_multiple_patterns(self, rel_yn):
+    def test_no_map_if_not_exist(self, rel_yn):
         aliases = PathAliases(relative=rel_yn)
+        aliases.add('/ned/home/*/src', './mysrc')
+        self.assert_unchanged(aliases, '/ned/home/foo/src/a.py', exists=False)
+        self.assert_unchanged(aliases, 'foo/src/a.py', exists=False)
+
+    def test_no_dotslash(self, rel_yn):
+        # The result shouldn't start with "./" if the map result didn't.
+        aliases = PathAliases(relative=rel_yn)
+        aliases.add('*/project', '.')
+        # Because the map result has no slash, the actual result is os-dependent.
+        self.assert_mapped(aliases, '/ned/home/project/src/a.py', f'src{os.sep}a.py')
+
+    def test_multiple_patterns(self, rel_yn):
+        # also test the debugfn...
+        msgs = []
+        aliases = PathAliases(debugfn=msgs.append, relative=rel_yn)
         aliases.add('/home/*/src', './mysrc')
         aliases.add('/lib/*/libsrc', './mylib')
-        self.assert_mapped(aliases, '/home/foo/src/a.py', './mysrc/a.py', relative=rel_yn)
-        self.assert_mapped(aliases, '/lib/foo/libsrc/a.py', './mylib/a.py', relative=rel_yn)
+        self.assert_mapped(aliases, '/home/foo/src/a.py', './mysrc/a.py')
+        self.assert_mapped(aliases, '/lib/foo/libsrc/a.py', './mylib/a.py')
+        if rel_yn:
+            assert msgs == [
+                "Aliases (relative=True):",
+                " Rule: '/home/*/src' -> './mysrc/' using regex " +
+                    "'[/\\\\\\\\]home[/\\\\\\\\][^/\\\\\\\\]*[/\\\\\\\\]src[/\\\\\\\\]'",
+                " Rule: '/lib/*/libsrc' -> './mylib/' using regex " +
+                    "'[/\\\\\\\\]lib[/\\\\\\\\][^/\\\\\\\\]*[/\\\\\\\\]libsrc[/\\\\\\\\]'",
+                "Matched path '/home/foo/src/a.py' to rule '/home/*/src' -> './mysrc/', " +
+                    "producing './mysrc/a.py'",
+                "Matched path '/lib/foo/libsrc/a.py' to rule '/lib/*/libsrc' -> './mylib/', " +
+                    "producing './mylib/a.py'",
+            ]
+        else:
+            assert msgs == [
+                "Aliases (relative=False):",
+                " Rule: '/home/*/src' -> './mysrc/' using regex " +
+                    "'[/\\\\\\\\]home[/\\\\\\\\][^/\\\\\\\\]*[/\\\\\\\\]src[/\\\\\\\\]'",
+                " Rule: '/lib/*/libsrc' -> './mylib/' using regex " +
+                    "'[/\\\\\\\\]lib[/\\\\\\\\][^/\\\\\\\\]*[/\\\\\\\\]libsrc[/\\\\\\\\]'",
+                "Matched path '/home/foo/src/a.py' to rule '/home/*/src' -> './mysrc/', " +
+                    f"producing {files.canonical_filename('./mysrc/a.py')!r}",
+                "Matched path '/lib/foo/libsrc/a.py' to rule '/lib/*/libsrc' -> './mylib/', " +
+                    f"producing {files.canonical_filename('./mylib/a.py')!r}",
+            ]
 
     @pytest.mark.parametrize("badpat", [
         "/ned/home/*",
@@ -317,7 +464,7 @@ class PathAliasesTest(CoverageTest):
         aliases = PathAliases(relative=rel_yn)
         aliases.add('/home/ned/*/src', './mysrc')
         aliases.add(r'c:\ned\src', './mysrc')
-        self.assert_mapped(aliases, r'C:\Ned\src\sub\a.py', './mysrc/sub/a.py', relative=rel_yn)
+        self.assert_mapped(aliases, r'C:\Ned\src\sub\a.py', './mysrc/sub/a.py')
 
         aliases = PathAliases(relative=rel_yn)
         aliases.add('/home/ned/*/src', r'.\mysrc')
@@ -326,42 +473,80 @@ class PathAliasesTest(CoverageTest):
             aliases,
             r'/home/ned/foo/src/sub/a.py',
             r'.\mysrc\sub\a.py',
-            relative=rel_yn,
         )
 
-    def test_windows_on_linux(self, rel_yn):
+    # Try the paths in both orders.
+    lin = "*/project/module/"
+    win = "*\\project\\module\\"
+    lin_win_paths = [[lin, win], [win, lin]]
+
+    @pytest.mark.parametrize("paths", lin_win_paths)
+    def test_windows_on_linux(self, paths, rel_yn):
         # https://github.com/nedbat/coveragepy/issues/618
-        lin = "*/project/module/"
-        win = "*\\project\\module\\"
+        aliases = PathAliases(relative=rel_yn)
+        for path in paths:
+            aliases.add(path, "project/module")
+        self.assert_mapped(
+            aliases,
+            "C:\\a\\path\\somewhere\\coveragepy_test\\project\\module\\tests\\file.py",
+            "project/module/tests/file.py",
+        )
 
-        # Try the paths in both orders.
-        for paths in [[lin, win], [win, lin]]:
-            aliases = PathAliases(relative=rel_yn)
-            for path in paths:
-                aliases.add(path, "project/module")
-            self.assert_mapped(
-                aliases,
-                "C:\\a\\path\\somewhere\\coveragepy_test\\project\\module\\tests\\file.py",
-                "project/module/tests/file.py",
-                relative=rel_yn,
-            )
-
-    def test_linux_on_windows(self, rel_yn):
+    @pytest.mark.parametrize("paths", lin_win_paths)
+    def test_linux_on_windows(self, paths, rel_yn):
         # https://github.com/nedbat/coveragepy/issues/618
-        lin = "*/project/module/"
-        win = "*\\project\\module\\"
+        aliases = PathAliases(relative=rel_yn)
+        for path in paths:
+            aliases.add(path, "project\\module")
+        self.assert_mapped(
+            aliases,
+            "C:/a/path/somewhere/coveragepy_test/project/module/tests/file.py",
+            "project\\module\\tests\\file.py",
+        )
 
-        # Try the paths in both orders.
-        for paths in [[lin, win], [win, lin]]:
-            aliases = PathAliases(relative=rel_yn)
-            for path in paths:
-                aliases.add(path, "project\\module")
-            self.assert_mapped(
-                aliases,
-                "C:/a/path/somewhere/coveragepy_test/project/module/tests/file.py",
-                "project\\module\\tests\\file.py",
-                relative=rel_yn,
-            )
+    @pytest.mark.parametrize("paths", lin_win_paths)
+    def test_relative_windows_on_linux(self, paths):
+        # https://github.com/nedbat/coveragepy/issues/991
+        aliases = PathAliases(relative=True)
+        for path in paths:
+            aliases.add(path, "project/module")
+        self.assert_mapped(
+            aliases,
+            r"project\module\tests\file.py",
+            r"project/module/tests/file.py",
+        )
+
+    @pytest.mark.parametrize("paths", lin_win_paths)
+    def test_relative_linux_on_windows(self, paths):
+        # https://github.com/nedbat/coveragepy/issues/991
+        aliases = PathAliases(relative=True)
+        for path in paths:
+            aliases.add(path, r"project\module")
+        self.assert_mapped(
+            aliases,
+            r"project/module/tests/file.py",
+            r"project\module\tests\file.py",
+        )
+
+    @pytest.mark.skipif(env.WINDOWS, reason="This test assumes Unix file system")
+    def test_implicit_relative_windows_on_linux(self):
+        # https://github.com/nedbat/coveragepy/issues/991
+        aliases = PathAliases(relative=True)
+        self.assert_mapped(
+            aliases,
+            r"project\module\tests\file.py",
+            r"project/module/tests/file.py",
+        )
+
+    @pytest.mark.skipif(not env.WINDOWS, reason="This test assumes Windows file system")
+    def test_implicit_relative_linux_on_windows(self):
+        # https://github.com/nedbat/coveragepy/issues/991
+        aliases = PathAliases(relative=True)
+        self.assert_mapped(
+            aliases,
+            r"project/module/tests/file.py",
+            r"project\module\tests\file.py",
+        )
 
     def test_multiple_wildcard(self, rel_yn):
         aliases = PathAliases(relative=rel_yn)
@@ -370,7 +555,6 @@ class PathAliasesTest(CoverageTest):
             aliases,
             '/home/jenkins/xx/a/yy/b/zz/django/foo/bar.py',
             './django/foo/bar.py',
-            relative=rel_yn,
         )
 
     def test_windows_root_paths(self, rel_yn):
@@ -380,29 +564,26 @@ class PathAliasesTest(CoverageTest):
             aliases,
             "X:\\a\\file.py",
             "/tmp/src/a/file.py",
-            relative=rel_yn,
         )
         self.assert_mapped(
             aliases,
             "X:\\file.py",
             "/tmp/src/file.py",
-            relative=rel_yn,
         )
 
     def test_leading_wildcard(self, rel_yn):
         aliases = PathAliases(relative=rel_yn)
         aliases.add('*/d1', './mysrc1')
         aliases.add('*/d2', './mysrc2')
-        self.assert_mapped(aliases, '/foo/bar/d1/x.py', './mysrc1/x.py', relative=rel_yn)
-        self.assert_mapped(aliases, '/foo/bar/d2/y.py', './mysrc2/y.py', relative=rel_yn)
+        self.assert_mapped(aliases, '/foo/bar/d1/x.py', './mysrc1/x.py')
+        self.assert_mapped(aliases, '/foo/bar/d2/y.py', './mysrc2/y.py')
 
-    # The root test case was added for the manylinux Docker images,
-    # and I'm not sure how it should work on Windows, so skip it.
-    cases = [".", "..", "../other"]
-    if not env.WINDOWS:
-        cases += ["/"]
-    @pytest.mark.parametrize("dirname", cases)
+    @pytest.mark.parametrize("dirname", [".", "..", "../other", "/"])
     def test_dot(self, dirname):
+        if env.WINDOWS and dirname == "/":
+            # The root test case was added for the manylinux Docker images,
+            # and I'm not sure how it should work on Windows, so skip it.
+            pytest.skip("Don't know how to handle root on Windows")
         aliases = PathAliases()
         aliases.add(dirname, '/the/source')
         the_file = os.path.join(dirname, 'a.py')
@@ -413,6 +594,19 @@ class PathAliasesTest(CoverageTest):
         self.assert_mapped(aliases, the_file, '/the/source/a.py')
 
 
+class PathAliasesRealFilesTest(CoverageTest):
+    """Tests for coverage/files.py:PathAliases using real files."""
+
+    def test_aliasing_zip_files(self):
+        self.make_file("src/zipfiles/code.zip", "fake zip, doesn't matter")
+        aliases = PathAliases()
+        aliases.add("*/d1", "./src")
+        aliases.add("*/d2", "./src")
+
+        expected = files.canonical_filename("src/zipfiles/code.zip/p1.py")
+        assert aliases.map("tox/d1/zipfiles/code.zip/p1.py") == expected
+
+
 class FindPythonFilesTest(CoverageTest):
     """Tests of `find_python_files`."""
 
@@ -425,13 +619,30 @@ class FindPythonFilesTest(CoverageTest):
         self.make_file("sub/ssub/~s.py")            # nope: editor effluvia
         self.make_file("sub/lab/exp.py")            # nope: no __init__.py
         self.make_file("sub/windows.pyw")
-        py_files = set(find_python_files("sub"))
+        py_files = set(find_python_files("sub", include_namespace_packages=False))
         self.assert_same_files(py_files, [
             "sub/a.py", "sub/b.py",
             "sub/ssub/__init__.py", "sub/ssub/s.py",
             "sub/windows.pyw",
         ])
 
+    def test_find_python_files_include_namespace_packages(self):
+        self.make_file("sub/a.py")
+        self.make_file("sub/b.py")
+        self.make_file("sub/x.c")                   # nope: not .py
+        self.make_file("sub/ssub/__init__.py")
+        self.make_file("sub/ssub/s.py")
+        self.make_file("sub/ssub/~s.py")            # nope: editor effluvia
+        self.make_file("sub/lab/exp.py")
+        self.make_file("sub/windows.pyw")
+        py_files = set(find_python_files("sub", include_namespace_packages=True))
+        self.assert_same_files(py_files, [
+            "sub/a.py", "sub/b.py",
+            "sub/ssub/__init__.py", "sub/ssub/s.py",
+            "sub/lab/exp.py",
+            "sub/windows.pyw",
+        ])
+
 
 @pytest.mark.skipif(not env.WINDOWS, reason="Only need to run Windows tests on Windows.")
 class WindowsFileTest(CoverageTest):
