@@ -3,6 +3,7 @@
 
 """Tests that HTML generation is awesome."""
 
+import collections
 import datetime
 import glob
 import json
@@ -52,7 +53,9 @@ class HtmlTestHelpers(CoverageTest):
         self.clean_local_file_imports()
         cov = coverage.Coverage(**(covargs or {}))
         self.start_import_stop(cov, "main_file")
-        return cov.html_report(**(htmlargs or {}))
+        ret = cov.html_report(**(htmlargs or {}))
+        self.assert_valid_hrefs()
+        return ret
 
     def get_html_report_content(self, module):
         """Return the content of the HTML report for `module`."""
@@ -64,7 +67,7 @@ class HtmlTestHelpers(CoverageTest):
     def get_html_index_content(self):
         """Return the content of index.html.
 
-        Timestamps are replaced with a placeholder so that clocks don't matter.
+        Time stamps are replaced with a placeholder so that clocks don't matter.
 
         """
         with open("htmlcov/index.html") as f:
@@ -82,19 +85,42 @@ class HtmlTestHelpers(CoverageTest):
         return index
 
     def assert_correct_timestamp(self, html):
-        """Extract the timestamp from `html`, and assert it is recent."""
+        """Extract the time stamp from `html`, and assert it is recent."""
         timestamp_pat = r"created at (\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2})"
         m = re.search(timestamp_pat, html)
-        assert m, "Didn't find a timestamp!"
+        assert m, "Didn't find a time stamp!"
         timestamp = datetime.datetime(*map(int, m.groups()))
-        # The timestamp only records the minute, so the delta could be from
+        # The time stamp only records the minute, so the delta could be from
         # 12:00 to 12:01:59, or two minutes.
         self.assert_recent_datetime(
             timestamp,
             seconds=120,
-            msg=f"Timestamp is wrong: {timestamp}",
+            msg=f"Time stamp is wrong: {timestamp}",
         )
 
+    def assert_valid_hrefs(self):
+        """Assert that the hrefs in htmlcov/*.html to see the references are valid.
+
+        Doesn't check external links (those with a protocol).
+        """
+        hrefs = collections.defaultdict(set)
+        for fname in glob.glob("htmlcov/*.html"):
+            with open(fname) as fhtml:
+                html = fhtml.read()
+            for href in re.findall(r""" href=['"]([^'"]*)['"]""", html):
+                if href.startswith("#"):
+                    assert re.search(rf""" id=['"]{href[1:]}['"]""", html), (
+                        f"Fragment {href!r} in {fname} has no anchor"
+                    )
+                    continue
+                if "://" in href:
+                    continue
+                hrefs[href].add(fname)
+        for href, sources in hrefs.items():
+            assert os.path.exists(f"htmlcov/{href}"), (
+                f"These files link to {href!r}, which doesn't exist: {', '.join(sources)}"
+            )
+
 
 class FileWriteTracker:
     """A fake object to track how `open` is used to write files."""
@@ -589,7 +615,7 @@ def compare_html(expected, actual, extra_scrubs=None):
     __tracebackhide__ = True    # pytest, please don't show me this function.
     scrubs = [
         (r'/coverage.readthedocs.io/?[-.\w/]*', '/coverage.readthedocs.io/VER'),
-        (r'coverage.py v[\d.abc]+', 'coverage.py vVER'),
+        (r'coverage.py v[\d.abcdev]+', 'coverage.py vVER'),
         (r'created at \d\d\d\d-\d\d-\d\d \d\d:\d\d [-+]\d\d\d\d', 'created at DATE'),
         (r'created at \d\d\d\d-\d\d-\d\d \d\d:\d\d', 'created at DATE'),
         # Occasionally an absolute path is in the HTML report.
@@ -1164,3 +1190,21 @@ class HtmlWithContextsTest(HtmlTestHelpers, CoverageTest):
                 if label == ld.contexts_label or label in (ld.contexts or ())
             ]
             assert sorted(expected) == sorted(actual)
+
+
+class HtmlHelpersTest(HtmlTestHelpers, CoverageTest):
+    """Tests of the helpers in HtmlTestHelpers."""
+
+    def test_bad_link(self):
+        # Does assert_valid_hrefs detect links to non-existent files?
+        self.make_file("htmlcov/index.html", "<a href='nothing.html'>Nothing</a>")
+        msg = "These files link to 'nothing.html', which doesn't exist: htmlcov.index.html"
+        with pytest.raises(AssertionError, match=msg):
+            self.assert_valid_hrefs()
+
+    def test_bad_anchor(self):
+        # Does assert_valid_hrefs detect fragments that go nowhere?
+        self.make_file("htmlcov/index.html", "<a href='#nothing'>Nothing</a>")
+        msg = "Fragment '#nothing' in htmlcov.index.html has no anchor"
+        with pytest.raises(AssertionError, match=msg):
+            self.assert_valid_hrefs()
