@@ -3,6 +3,7 @@
 
 """Tests for coverage.py's code parsing."""
 
+import ast
 import os.path
 import textwrap
 import warnings
@@ -11,10 +12,10 @@ import pytest
 
 from coverage import env
 from coverage.exceptions import NotPython
-from coverage.parser import ast_dump, ast_parse, PythonParser
+from coverage.parser import ast_dump, PythonParser
 
 from tests.coveragetest import CoverageTest, TESTS_DIR
-from tests.helpers import arcz_to_arcs, re_lines
+from tests.helpers import arcz_to_arcs, re_lines, xfail_pypy_3749
 
 
 class PythonParserTest(CoverageTest):
@@ -43,7 +44,7 @@ class PythonParserTest(CoverageTest):
                 pass
             """)
         assert parser.exit_counts() == {
-            2:2, 3:1, 4:2, 5:1, 7:1, 9:2, 10:1
+            2:1, 3:1, 4:2, 5:1, 7:1, 9:1, 10:1
         }
 
     def test_generator_exit_counts(self):
@@ -89,7 +90,7 @@ class PythonParserTest(CoverageTest):
                     pass
             """)
         assert parser.exit_counts() == {
-            1:1, 2:1, 3:1
+            1:0, 2:1, 3:1
         }
 
     def test_missing_branch_to_excluded_code(self):
@@ -139,10 +140,7 @@ class PythonParserTest(CoverageTest):
                 '''
                 """)
 
-    @pytest.mark.xfail(
-        env.PYPY and env.PYPYVERSION == (7, 3, 0),
-        reason="https://bitbucket.org/pypy/pypy/issues/3139",
-    )
+    @xfail_pypy_3749
     def test_decorator_pragmas(self):
         parser = self.parse_source("""\
             # 1
@@ -178,6 +176,7 @@ class PythonParserTest(CoverageTest):
         assert parser.raw_statements == raw_statements
         assert parser.statements == {8}
 
+    @xfail_pypy_3749
     def test_decorator_pragmas_with_colons(self):
         # A colon in a decorator expression would confuse the parser,
         # ending the exclusion of the decorated function.
@@ -212,6 +211,7 @@ class PythonParserTest(CoverageTest):
         assert parser.raw_statements == {1, 2, 3, 5, 6, 7, 8}
         assert parser.statements == {1, 2, 3}
 
+    @xfail_pypy_3749
     def test_empty_decorated_function(self):
         parser = self.parse_source("""\
             def decorator(func):
@@ -248,6 +248,15 @@ class PythonParserTest(CoverageTest):
         assert expected_arcs == parser.arcs()
         assert expected_exits == parser.exit_counts()
 
+    def test_fuzzed_double_parse(self):
+        # https://bugs.chromium.org/p/oss-fuzz/issues/detail?id=50381
+        # The second parse used to raise `TypeError: 'NoneType' object is not iterable`
+        msg = "EOF in multi-line statement"
+        with pytest.raises(NotPython, match=msg):
+            self.parse_source("]")
+        with pytest.raises(NotPython, match=msg):
+            self.parse_source("]")
+
 
 class ParserMissingArcDescriptionTest(CoverageTest):
     """Tests for PythonParser.missing_arc_description."""
@@ -464,7 +473,7 @@ class ParserFileTest(CoverageTest):
             class Bar:
                 pass
             """
-        counts = { 2:2, 3:1, 4:2, 5:1, 7:1, 9:2, 10:1 }
+        counts = { 2:1, 3:1, 4:2, 5:1, 7:1, 9:1, 10:1 }
         fname = slug + ".py"
         self.make_file(fname, text, newline=newline)
         parser = self.parse_file(fname)
@@ -522,7 +531,7 @@ def test_ast_dump():
             with warnings.catch_warnings():
                 # stress_phystoken.tok has deprecation warnings, suppress them.
                 warnings.filterwarnings("ignore", message=r".*invalid escape sequence",)
-                ast_root = ast_parse(source)
+                ast_root = ast.parse(source)
         result = []
         ast_dump(ast_root, print=result.append)
         if num_lines < 100:
